<?php
/*
 * Plugin Name: SVG Logo and Text Effects Title Replacement extension
 * Description: Extends the WordPress SLATE plugin to allow replacing the site title and page / post titles with SVG text.
 * Version: 1.0.0
 * Plugin URI: http://www.dashed-slug.net/svg-logo-and-text-effects-extensions/titles-replacement-extension
 * Author: Dashed Slug <info@dashed-slug.net>
 * Author URI: http://dashed-slug.net
 * Text Domain: slate-titles
 * Domain Path: /languages/
 * License: GPLv2 or later
 *
 * @package slate-titles
 * @since 1.0.0
 */

// don't load directly
defined( 'ABSPATH' ) || die( '-1' );

include __DIR__ . '/ds-update.php';

if ( ! class_exists( 'SVG_Logo_and_Text_Effects_titles_replacement' ) ) {

	final class SVG_Logo_and_Text_Effects_titles_replacement {

		const OPT_RASTER_FALLBACK = 'slate-opt-raster-fallback';

		private static $_instance = null;

		private static $_errors = array();

		private $_in_body = false;

		private $_shapes = array();

		private $_fonts = array();

		private $_filters = array();

		private $_fills = array();

		private $_presets = array();

		private function __clone() {
			// Cloning disabled
		}

		/**
		 * Creates or returns an instance of this class.
		 *
		 * @return SVG_Logo_and_Text_Effects_titles_replacement
		 */
		public static function get_instance() {
			if ( ! ( self::$_instance instanceof self ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public static function error_handler( $errno, $errstr, $errfile, $errline, $errcontext ) {
			if ( ! ( error_reporting() & $errno ) ) {
				return;
			}

			// append error to show on admin_notices
			self::$_errors[] = array(
				'errno' => $errno,
				'errstr' => $errstr,
				'errfile' => $errfile,
				'errline' => $errline,
				'errcontext' => $errcontext,
				'current_filter' => current_filter() );

			error_log( __CLASS__ . ": errno: $errno, errstr: $errstr, errfile: $errfile, errline: $errline" );
		}

		private function __construct() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			// Show admin notices
			add_action( 'admin_notices', array( &$this, 'action_admin_notices' ) );

			// Title-related hooks
			add_action( 'init', array( &$this, 'action_init' ) );
			add_action( 'wp_head', array( &$this, 'action_wp_head_finished' ), PHP_INT_MAX );
			add_action( 'wp_footer', array( &$this, 'action_wp_footer_started' ), 0 );
			add_action( 'customize_register', array( &$this, 'action_customize_register' ) );

			restore_error_handler();
		}

		public function action_init() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			if ( class_exists( 'SVG_Logo_and_Text_Effects') ) {

				$slate = SVG_Logo_and_Text_Effects::get_instance();

				foreach ( array( 'shapes', 'fonts', 'filters', 'fills', 'presets') as $resource_type ) {
					$this->{"_$resource_type"} = $slate->{"get_$resource_type"}();
				}
			}
			restore_error_handler();
		}

		public function action_wp_head_finished() {
			$this->_in_body = true;

			if ( get_option( 'slate_site_title_filter_bloginfo' ) ) {
				add_filter( 'bloginfo', array( &$this, 'filter_site_title' ), PHP_INT_MAX, 2 );
			}

			if ( get_option( 'slate_page_title_filter_the_title' ) ) {
				add_filter( 'the_title', array( &$this, 'filter_page_title' ) );
			}

			if ( get_option( 'slate_page_title_filter_single_post_title' ) ) {
				add_filter( 'single_post_title', array( &$this, 'filter_page_title' ) );
			}

		}

		public function action_wp_footer_started() {
			$this->_in_body = false;
		}

		public function in_body() {
			return $this->_in_body;
		}

		/**
		 * Adds settings to the customizer api
		 *
		 * @param unknown $wp_customize
		 */
		public function action_customize_register( $wp_customize ) {
			if ( method_exists( $wp_customize, 'add_panel' ) ) {
				$wp_customize->add_panel(
					"slate_site_title_panel",
					array(
						'priority' => 10,
						'capability' => 'edit_theme_options',
						'theme_supports' => '',
						'title' => __( 'Site SVG Title', 'slate'  ),
						'description' => __( 'SVG settings for the site title', 'slate'  ) ) );

				$wp_customize->add_panel(
					"slate_page_title_panel",
					array(
						'priority' => 10,
						'capability' => 'edit_theme_options',
						'theme_supports' => '',
						'title' => __( 'Page/Post SVG Title', 'slate'  ),
						'description' => __( 'SVG settings for page and post titles', 'slate'  ) ) );

				foreach ( array( 'site_title', 'page_title' ) as $title_type ) {

					// SHAPE vars section
					$wp_customize->add_section(
						"slate_{$title_type}_section_shape",
						array(
							'title' => __( 'Shape', 'slate' ),
							'description' => __( 'Choose the shape SVG template that your text will be based on. Here you can also tweak the parameters that are specific to this shape. After choosing a shape, you can then go back and tweak all the other SVG parameters. Or set to (none) to disable SVG rendering.' ),
							'priority' => 30,
							'panel' => "slate_{$title_type}_panel" ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_shape",
						array(
							'default' => '',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						"slate_{$title_type}_shape",
						array(
							'label' => __( 'Text Shape', 'slate' ),
							'description' => __( 'Choose a shape SVG template', 'slate' ),
							'type' => 'select',
							'settings' => "slate_{$title_type}_shape",
							'choices' => array_merge(
								array( '' => __( '(none)', 'slate'  ) ),
								array_combine( array_keys( $this->_shapes ), array_keys( $this->_shapes ) ) ),
							'section' => "slate_{$title_type}_section_shape" ) );

					foreach ( $this->_shapes as $shape_name => $shape ) {

						foreach ( $shape['params_defs'] as $param_name => $param_def ) {
							if ( '_' != substr( $param_name, 0, 1 ) ) {

								$wp_customize->add_setting(
									"slate_{$title_type}_{$shape_name}_{$param_name}",
									array(
										'default' => $param_def['default'],
										'type' => 'option',
										'capability' => 'edit_theme_options' ) );

								$control = array(
									'label' => "'{$shape_name}' shape variable: {$param_def['name']}",
									'description' => "(Default:{$param_def['default']}) {$param_def['description']}",
									'settings' => "slate_{$title_type}_{$shape_name}_{$param_name}",
									'section' => "slate_{$title_type}_section_shape",
									'active_callback' => array( &$this, "{$title_type}_shape_control_is_active" ) );

								if ( 'color' == $param_def['type'] ) {
									$wp_customize->add_control(
										new WP_Customize_Color_Control(
											$wp_customize,
											"slate_{$title_type}_{$shape_name}_{$param_name}",
											$control ) );
								} else {
									$wp_customize->add_control( "slate_{$title_type}_{$shape_name}_{$param_name}", $control );
								}
							}
						} // end foreach shape template params
					} // end foreach shape templates

					// TEXT section
					$wp_customize->add_section(
						"slate_{$title_type}_section_text",
						array(
							'title' => __( 'Text', 'slate' ),
							'description' => __( 'Here you can tweak general text parameters that apply to any shape', 'slate' ),
							'priority' => 30,
							'panel' => "slate_{$title_type}_panel" ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_font_family",
						array(
							'default' => 'Open Sans',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						"slate_{$title_type}_font_family",
						array(
							'label' => __( 'Font Family', 'slate' ),
							'description' => __(
								'Choose a font family to use for your text, or clear to use the surrounding text\'s font',
								'slate' ),
							'type' => 'select',
							'settings' => "slate_{$title_type}_font_family",
							'choices' => array_merge(
								array( '' => __( '(none)', 'slate'  ) ),
								array_combine( array_keys( $this->_fonts ), array_keys( $this->_fonts ) ) ),
								'section' => "slate_{$title_type}_section_text",
								'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_font_size",
						array(
							'default' => '',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						new WP_Customize_Control(
							$wp_customize,
							"slate_{$title_type}_font_size",
							array(
								'label' => __( 'Font Size', 'slate' ),
								'description' => __(
									'Enter a font size. You may use any valid SVG font sizes, such as "12px", "14pt", "120%", etc',
									'slate' ),
								'type' => 'text',
								'settings' => "slate_{$title_type}_font_size",
								'section' => "slate_{$title_type}_section_text",
								'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) ) ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_stroke",
						array(
							'default' => '',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						new WP_Customize_Color_Control(
							$wp_customize,
							"slate_{$title_type}_stroke",
							array(
								'label' => __( 'Stroke Color', 'slate' ),
								'description' => __( 'Choose a stroke color. ' .
									'This will be the color of the outline of the letters.', 'slate' ),
								'settings' => "slate_{$title_type}_stroke",
								'section' => "slate_{$title_type}_section_text",
								'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) ) ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_stroke_width",
						array(
							'default' => '1px',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						"slate_{$title_type}_stroke_width",
						array(
							'label' => __( 'Stroke Width', 'slate' ),
							'description' => __(
								'Enter a width in pixels for the text\'s outline. This can be a decimal number, e.g. 0.5',
								'slate' ),
							'type' => 'text',
							'settings' => "slate_{$title_type}_stroke_width",
							'section' => "slate_{$title_type}_section_text",
							'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_skew_x_deg",
						array(
							'default' => '0',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						"slate_{$title_type}_skew_x_deg",
						array(
							'label' => __( 'X-axis skew angle', 'slate' ),
							'description' => __( 'Enter an X-axis skew angle in degrees. e.g. "-45" or "20". Warning: CSS 3D transforms might not work on Firefox with some themes. Please check before using.', 'slate' ),
							'type' => 'text',
							'settings' => "slate_{$title_type}_skew_x_deg",
							'section' => "slate_{$title_type}_section_text",
							'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) ) );

					// FILL vars section
					$wp_customize->add_section(
						"slate_{$title_type}_section_fill",
						array(
							'title' => __( 'Fill', 'slate' ),
							'description' => __(
								'Here you can choose a fill pattern and tweak its parameters. ' .
								'The fill pattern will be used to render the inside of the letters.' ),
							'priority' => 30,
							'panel' => "slate_{$title_type}_panel" ) );

					$wp_customize->add_setting(
						"slate_{$title_type}_fill",
						array(
							'default' => '',
							'type' => 'option',
							'capability' => 'edit_theme_options' ) );

					$wp_customize->add_control(
						"slate_{$title_type}_fill",
						array(
							'label' => __( 'Text Fill', 'slate' ),
							'description' => __( 'Choose a pattern for the text\'s fill', 'slate' ),
							'type' => 'select',
							'settings' => "slate_{$title_type}_fill",
							'choices' => array_merge(
								array( '' => __( '(none)', 'slate'  ) ),
								array_combine( array_keys( $this->_fills ), array_keys( $this->_fills ) ) ),
							'section' => "slate_{$title_type}_section_fill",
							'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) ) );

					foreach ( $this->_fills as $fill_name => $fill ) {
						foreach ( $fill['params_defs'] as $param_name => $param_def ) {
							if ( '_' != substr( $param_name, 0, 1 ) ) {
								$wp_customize->add_setting(
									"slate_{$title_type}_fill_{$fill_name}_{$param_name}",
									array(
										'default' => $param_def['default'],
										'type' => 'option',
										'capability' => 'edit_theme_options' ) );

								$control = array(
									'label' => "'{$fill_name}' fill variable: {$param_def['name']}",
									'description' => "(Default:{$param_def['default']}) {$param_def['description']}",
									'settings' => "slate_{$title_type}_fill_{$fill_name}_{$param_name}",
									'section' => "slate_{$title_type}_section_fill",
									'active_callback' => array( &$this, "{$title_type}_fill_control_is_active" ) );

								if ( 'color' == $param_def['type'] ) {

									$wp_customize->add_control(
										new WP_Customize_Color_Control(
											$wp_customize,
											"slate_{$title_type}_{$fill_name}_{$param_name}",
											$control ) );
								} else {
									$wp_customize->add_control(
										"slate_{$title_type}_fill_{$fill_name}_{$param_name}",
										$control );
								}
							}
						} // end foreach fill template params
					} // end foreach fill templates

					// FILTER vars section
					foreach ( $this->_filters as $filter_name => $filter ) {
						$wp_customize->add_section(
							"slate_{$title_type}_section_filter_{$filter_name}",
							array(
								'title' => sprintf( __( "'%s' filter", 'slate' ), $filter_name ),
								'description' => sprintf( __( 'Here you can tweak the parameters for the %s filter. ' .
									'Enter values only if you want this filter enabled. ' .
									'Leaving the parameters at their default values will disable the filter.' ), $filter_name ),
								'priority' => 30,
								'panel' => "slate_{$title_type}_panel" ) );

						foreach ( $filter['params_defs'] as $param_name => $param_def ) {
							if ( '_' != substr( $param_name, 0, 1 ) ) {
								$wp_customize->add_setting(
									"slate_{$title_type}_filter_{$filter_name}_{$param_name}",
									array(
										'default' => $param_def['default'],
										'type' => 'option',
										'capability' => 'edit_theme_options' ) );

								$control = array(
									'label' => "'{$filter_name}' filter variable: {$param_def['name']}",
									'description' => "(Default:{$param_def['default']}) {$param_def['description']}",
									'settings' => "slate_{$title_type}_filter_{$filter_name}_{$param_name}",
									'section' => "slate_{$title_type}_section_filter_{$filter_name}",
									'active_callback' => array( &$this, "{$title_type}_is_valid_shape_selected" ) );

								if ( 'color' == $param_def['type'] ) {
									$wp_customize->add_control(
										new WP_Customize_Color_Control(
											$wp_customize,
											"slate_{$title_type}_filter_{$filter_name}_{$param_name}",
											$control ) );
								} else {
									$wp_customize->add_control(
										"slate_{$title_type}_filter_{$filter_name}_{$param_name}",
										$control );
								}
							}
						} // end foreach filter template params
					} // end foreach filter templates
				} // end foreach title type

				// THEME INTEGRATION section for site title
				$wp_customize->add_section(
					"slate_site_title_section_int",
					array(
						'title' => __( 'Theme Integration', 'slate' ),
						'description' => __( 'Tweak this to achieve compatibility with your theme. ' .
							'If you experience issues with your theme, consult the accompanying manual ' .
							'for troubleshooting instructions.', 'slate' ),
						'priority' => 30,
						'panel' => "slate_site_title_panel" ) );

				$wp_customize->add_setting(
					"slate_site_title_filter_bloginfo",
					array(
						'default' => '',
						'type' => 'option',
						'capability' => 'edit_theme_options' ) );

				$wp_customize->add_control(
					"slate_site_title_filter_bloginfo",
					array(
						'label' => __( 'bloginfo hook', 'slate' ),
						'description' => __( 'Enable this hook to filter calls to ' .
							'<code>bloginfo( "name" )</code> and ' .
							'<code>get_bloginfo( "name", "display" )</code>', 'slate' ),
						'type' => 'checkbox',
						'settings' => "slate_site_title_filter_bloginfo",
						'section' => "slate_site_title_section_int",
						'active_callback' => array( &$this, "site_title_is_valid_shape_selected" ) ) );


				// THEME INTEGRATION section for page title
				$wp_customize->add_section(
					"slate_page_title_section_int",
					array(
						'title' => __( 'Theme Integration', 'slate' ),
						'description' => __( 'Tweak these to achieve compatibility with your theme. If you experience issues with your theme, consult the accompanying manual for troubleshooting instructions.', 'slate' ),
						'priority' => 30,
						'panel' => "slate_page_title_panel" ) );

				$wp_customize->add_setting(
					"slate_page_title_filter_the_title",
					array(
						'default' => '',
						'type' => 'option',
						'capability' => 'edit_theme_options' ) );

				$wp_customize->add_control(
					"slate_page_title_filter_the_title",
					array(
						'label' => __( 'the_title hook', 'slate' ),
						'description' => __( 'Enable this hook to filter calls to <code>the_title()</code> or <code>get_the_title()</code>', 'slate' ),
						'type' => 'checkbox',
						'settings' => "slate_page_title_filter_the_title",
						'section' => "slate_page_title_section_int",
						'active_callback' => array( &$this, "page_title_is_valid_shape_selected" ) ) );

				$wp_customize->add_setting(
					"slate_page_title_filter_single_post_title",
					array(
						'default' => '',
						'type' => 'option',
						'capability' => 'edit_theme_options' ) );

				$wp_customize->add_control(
					"slate_page_title_filter_single_post_title",
					array(
						'label' => __( 'single_post_title hook', 'slate' ),
						'description' => __( 'Enable this hook to filter calls to <code>single_post_title()</code>, <code>wp_title()</code>, <code>wp_title_parts()</code>, <code>wp_get_document_title()</code>, etc.', 'slate' ),
						'type' => 'checkbox',
						'settings' => "slate_page_title_filter_single_post_title",
						'section' => "slate_page_title_section_int",
						'active_callback' => array( &$this, "page_title_is_valid_shape_selected" ) ) );
			}
		}

		/**
		 *
		 * @param WP Customize Control $control The WP control
		 * @param string $title_type One of 'site_title' or 'page_title'
		 * @param string $template_type One of 'shape', 'fill', or 'filter
		 * @param string $current_template_name 'shadow', 'color', etc...
		 */
		private function control_is_active( $control, $title_type, $template_type, $current_template_name ) {
			if ( ! $this->{ "{$title_type}_is_valid_shape_selected" }( $control ) ) {
				return false;
			}

			preg_match( "/^'(\S+)' $template_type variable: /", $control->label, $matches );
			if ( $matches ) {
				$template_name = $matches[1];

				if ( $current_template_name == $template_name ) {
					return true;
				}
			}
			return false;
		}

		public function site_title_is_valid_shape_selected( $control ) {
			$shape_setting = $control->manager->get_setting( "slate_site_title_shape" );
			if ( !is_object( $shape_setting ) ) {
				return false;
			}
			return array_key_exists( $shape_setting->value(), $this->_shapes );
		}

		public function page_title_is_valid_shape_selected( $control ) {
			$shape_setting = $control->manager->get_setting( "slate_page_title_shape" );
			if ( !is_object( $shape_setting ) ) {
				return false;
			}
			return array_key_exists( $shape_setting->value(), $this->_shapes );
		}

		public function site_title_shape_control_is_active( $control ) {
			$shape_setting = $control->manager->get_setting( "slate_site_title_shape" );
			if ( !is_object( $shape_setting ) ) {
				return false;
			}
			return $this->control_is_active( $control, 'site_title', 'shape', $shape_setting->value() );
		}

		public function page_title_shape_control_is_active( $control ) {
			$shape_setting = $control->manager->get_setting( "slate_page_title_shape" );
			if ( !is_object( $shape_setting ) ) {
				return false;
			}
			return $this->control_is_active( $control, 'page_title', 'shape', $shape_setting->value() );
		}

		public function site_title_fill_control_is_active( $control ) {
			$fill_setting = $control->manager->get_setting( "slate_site_title_fill" );
			if ( !is_object( $fill_setting ) ) {
				return false;
			}
			return $this->control_is_active( $control, 'site_title', 'fill', $fill_setting->value() );
		}

		public function page_title_fill_control_is_active( $control ) {
			$fill_setting = $control->manager->get_setting( "slate_page_title_fill" );
			if ( !is_object( $fill_setting ) ) {
				return false;
			}
			return $this->control_is_active( $control, 'page_title', 'fill', $fill_setting->value() );
		}

		/**
		 * Filters site title
		 */
		public function filter_site_title( $title, $show = null ) {
			if ( ! $this->_in_body || ! $title || 'name' != $show || is_feed() || is_admin() ) {
				return $title;
			}

			$atts = $this->_get_atts_for( 'site_title' );
			if ( $atts ) {
				$shape_name = get_option( "slate_site_title_shape" );
				return SVG_Logo_and_Text_Effects::get_instance()->shortcode( $atts, $title, "slate_$shape_name" );
			}
			return $title;
		}

		/**
		 * Filters page/post titles
		 *
		 * @param unknown $title
		 */
		public function filter_page_title( $title ) {
			if ( ! $this->_in_body || is_feed() || is_admin() || ! $title ) {
				return $title;
			}

			$atts = $this->_get_atts_for( 'page_title' );
			if ( $atts ) {
				$shape_name = get_option( "slate_page_title_shape" );
				return SVG_Logo_and_Text_Effects::get_instance()->shortcode( $atts, $title, "slate_$shape_name" );
			}
			return $title;
		}

		public function filter_body_class( $classes ) {
			// Only add classes on frontend.
			if ( is_admin() ) {
				return $classes;
			}

			// Add class to indicate that we're in the customizer.
			// Needed to address https://core.trac.wordpress.org/ticket/35824
			if ( isset( $_REQUEST['wp_customize'] ) ) {
				$classes[] = 'slate-customizer';
			}

			return $classes;
		}

		/**
		 * @param $title_type string One of 'page_title' or 'site_title'
		 * @return Array of attributes to pass to shortcode()
		 */
		private function _get_atts_for( $title_type ) {
			$shape_name = get_option( "slate_{$title_type}_shape");
			if ( ! $shape_name ) {
				return false;
			}

			$atts = array();

			// Text vars
			foreach ( array( '_font_family', '_font_size', '_stroke', '_stroke_width', '_skew_x_deg' ) as $general_var ) {
				$atts[ $general_var ] = get_option( "slate_{$title_type}{$general_var}");
			}

			// Shape vars
			if ( array_key_exists( $shape_name, $this->_shapes ) ) {
				foreach ( $this->_shapes[ $shape_name ]['params_defs'] as $param_name => $param_def ) {
					$val = get_option( "slate_{$title_type}_shape_{$shape_name}_{$param_name}", $param_def['default'] );

					if ( $val != $param_def['default'] ) {
						$atts[ $param_name ];
					}
				}

				// Fill vars
				$fill_name = $atts['fill'] = get_option( "slate_{$title_type}_fill" );
				if ( $fill_name && array_key_exists( $fill_name, $this->_fills ) ) {
					foreach ( $this->_fills[ $fill_name ]['params_defs'] as $param_name => $param_def ) {
						$val = get_option(
							"slate_{$title_type}_fill_{$fill_name}_{$param_name}",
							$param_def['default'] );

						if ( $val != $param_def['default'] ) {
							$atts["fill_{$fill_name}_{$param_name}"] = $val;
						}
					}
				}

				// Filter vars
				foreach ( $this->_filters as $filter_name => $filter ) {
					foreach ( $this->_filters[ $filter_name ]['params_defs'] as $param_name => $param_def ) {
						$val = get_option(
							"slate_{$title_type}_filter_{$filter_name}_{$param_name}",
							$param_def['default'] );

						if ( $val != $param_def['default'] ) {
							$atts["filter_{$filter_name}_{$param_name}"] = $val;
						}
					}
				}

				return $atts;
			}
			return false;
		}

		public function action_admin_notices() {
			set_error_handler( array( __CLASS__, 'error_handler' ) );

			if ( ! is_plugin_active( 'slate/slate.php' ) ) {
				echo '<div class="notice notice-warn is-dismissible"><p>' . sprintf(
					__( '<strong>%s</strong> requires the <strong><a href="http://dashed-slug.net/svg-logo-and-text-effects-wordpress-plugin" target="_blank">SVG Logo and Text Effects</a></strong> plugin. Please install and activate it on your site.', 'slate-vc' ),
					'SVG Logo and Text Effects Title Replacement extension' ) . '</p></div>';
			}

			foreach ( self::$_errors as $error ) {
				$level = 'notice';
				if ( E_USER_ERROR == $error['errno'] ) {
					$level = 'error';
				} elseif ( E_USER_WARNING == $error['errno'] ) {
					$level = 'warning';
				}

				echo "<div class=\"notice notice-$level is-dismissible\">";
				echo "<p style=\"font-size: larger;\">{$error['errstr']}</p><pre>" . 'SVG Logo and Text Effects Title Replacement extension' .
					" on action <code>{$error['current_filter']}</code>, <code>{$error['errfile']}</code>, line <code>{$error['errline']}</code>.</pre></div>";
			}
			restore_error_handler();
		}
	}
}

SVG_Logo_and_Text_Effects_titles_replacement::get_instance();

if ( ! function_exists( 'slate_single_post_title' ) ) {
	/**
	 * The page or post title is returned as SVG text, if enabled in customizer.
	 * The arguments and result are equivalent to those of single_post_title() from WordPress core.
	 */
	function slate_single_post_title( $prefix = '', $display = true ) {
		static $slate = null;

		if ( is_null( $slate ) ) {
			$slate = SVG_Logo_and_Text_Effects::get_instance();
		}

		$post = get_queried_object();

		if ( ! isset( $post->post_title ) )
			return '';

			$title = $prefix . $post->post_title;

			if ( ! get_option( 'slate_site_title_filter_single_post_title' ) ) {
				$title = $slate->filter_page_title( $title );
			}

			if ( $display ) {
				echo $title;
			} else {
				return $title;
			}
	}
}

if ( ! function_exists( 'slate_the_title' ) ) {
	/**
	 * The page or post title is returned as SVG text, if enabled in customizer.
	 * The arguments and result are equivalent to those of the_title() from WordPress core.
	 */
	function slate_the_title( $before = '', $after = '', $echo = true ) {
		static $slate = null;

		if ( is_null( $slate ) ) {
			$slate = SVG_Logo_and_Text_Effects::get_instance();
		}

		$title = get_the_title( );

		if ( strlen($title) == 0 ) {
			return;
		}

		$title = $before . $title . $after;

		if ( ! get_option( 'slate_site_title_filter_the_title' ) ) {
			$title = $slate->filter_page_title( $title );
		}

		if ( $echo ) {
			echo $title;
		} else {
			return $title;
		}
	}
}

if ( ! function_exists( 'slate_get_the_title' ) ) {
	/**
	 * The page or post title is returned as SVG text, if enabled in customizer.
	 * The arguments and result are equivalent to those of get_the_title() from WordPress core.
	 */
	function slate_get_the_title( $post = 0 ) {
		static $slate = null;

		if ( is_null( $slate ) ) {
			$slate = SVG_Logo_and_Text_Effects::get_instance();
		}

		$title = get_the_title( $post );

		if ( get_option( 'slate_site_title_filter_the_title' ) ) {

			return $title;

		} else {

			return $slate->filter_page_title( $title );

		}
	}
}

if ( ! function_exists( 'slate_bloginfo' ) ) {
	/**
	 * When called with $show == 'name', the site title is returned as SVG text, if enabled in customizer.
	 * For any other value of $show, the result is the same as that of bloginfo().
	 */
	function slate_bloginfo( $show = null ) {
		static $slate = null;

		if ( is_null( $slate ) ) {
			$slate = SVG_Logo_and_Text_Effects::get_instance();
		}

		$title = get_bloginfo( $show, 'display' );

		if ( get_option( 'slate_site_title_filter_bloginfo' ) ) {

			echo $title;

		} else {

			echo $slate->filter_site_title( $title, $show );

		}
	}
}

if ( ! function_exists( 'slate_get_bloginfo' ) ) {
	/**
	 * When called with $show == 'name', $filter == 'display', the site title is returned as SVG text, if enabled in customizer.
	 * For any other value of $show, the result is the same as that of get_bloginfo().
	 */
	function slate_get_bloginfo( $show = null, $filter = 'raw' ) {
		static $slate = null;

		if ( is_null( $slate ) ) {
			$slate = SVG_Logo_and_Text_Effects::get_instance();
		}

		$title = get_bloginfo( $show, $filter );

		if ( 'name' != $show || 'display' != $filter ) {
			return $title;
		}

		if ( get_option( 'slate_site_title_filter_bloginfo' ) ) {

			return $title;

		} else {

			return $slate->filter_site_title( $title, $show );

		}
	}
}

